import * as _codemirror_state from '@codemirror/state';
import { EditorState, TransactionSpec, Transaction, StateCommand, Facet, Extension, StateEffect } from '@codemirror/state';
import { EditorView, Rect, KeyBinding, Command } from '@codemirror/view';
import * as _lezer_common from '@lezer/common';

interface CompletionConfig {
    activateOnTyping?: boolean;
    selectOnOpen?: boolean;
    override?: readonly CompletionSource[] | null;
    closeOnBlur?: boolean;
    maxRenderedOptions?: number;
    defaultKeymap?: boolean;
    aboveCursor?: boolean;
    tooltipClass?: (state: EditorState) => string;
    optionClass?: (completion: Completion) => string;
    icons?: boolean;
    addToOptions?: {
        render: (completion: Completion, state: EditorState) => Node | null;
        position: number;
    }[];
    positionInfo?: (view: EditorView, list: Rect, option: Rect, info: Rect, space: Rect) => {
        style?: string;
        class?: string;
    };
    compareCompletions?: (a: Completion, b: Completion) => number;
    interactionDelay?: number;
}

interface Completion {
    label: string;
    detail?: string;
    info?: string | ((completion: Completion) => (Node | null | Promise<Node | null>));
    apply?: string | ((view: EditorView, completion: Completion, from: number, to: number) => void);
    type?: string;
    boost?: number;
    section?: string | CompletionSection;
}
interface CompletionSection {
    name: string;
    header?: (section: CompletionSection) => HTMLElement;
    rank?: number;
}
declare class CompletionContext {
    readonly state: EditorState;
    readonly pos: number;
    readonly explicit: boolean;
    constructor(state: EditorState, pos: number, explicit: boolean);
    tokenBefore(types: readonly string[]): {
        from: number;
        to: number;
        text: string;
        type: _lezer_common.NodeType;
    } | null;
    matchBefore(expr: RegExp): {
        from: number;
        to: number;
        text: string;
    } | null;
    get aborted(): boolean;
    addEventListener(type: "abort", listener: () => void): void;
}
declare function completeFromList(list: readonly (string | Completion)[]): CompletionSource;
declare function ifIn(nodes: readonly string[], source: CompletionSource): CompletionSource;
declare function ifNotIn(nodes: readonly string[], source: CompletionSource): CompletionSource;
type CompletionSource = (context: CompletionContext) => CompletionResult | null | Promise<CompletionResult | null>;
interface CompletionResult {
    from: number;
    to?: number;
    options: readonly Completion[];
    validFor?: RegExp | ((text: string, from: number, to: number, state: EditorState) => boolean);
    filter?: boolean;
    getMatch?: (completion: Completion) => readonly number[];
    update?: (current: CompletionResult, from: number, to: number, context: CompletionContext) => CompletionResult | null;
}
declare const pickedCompletion: _codemirror_state.AnnotationType<Completion>;
declare function insertCompletionText(state: EditorState, text: string, from: number, to: number): TransactionSpec;

declare function snippet(template: string): (editor: {
    state: EditorState;
    dispatch: (tr: Transaction) => void;
}, completion: Completion, from: number, to: number) => void;
declare const clearSnippet: StateCommand;
declare const nextSnippetField: StateCommand;
declare const prevSnippetField: StateCommand;
declare const snippetKeymap: Facet<readonly KeyBinding[], readonly KeyBinding[]>;
declare function snippetCompletion(template: string, completion: Completion): Completion;

declare function moveCompletionSelection(forward: boolean, by?: "option" | "page"): Command;
declare const acceptCompletion: Command;
declare const startCompletion: Command;
declare const closeCompletion: Command;

declare const completeAnyWord: CompletionSource;

interface CloseBracketConfig {
    brackets?: string[];
    before?: string;
    stringPrefixes?: string[];
}
declare function closeBrackets(): Extension;
declare const deleteBracketPair: StateCommand;
declare const closeBracketsKeymap: readonly KeyBinding[];
declare function insertBracket(state: EditorState, bracket: string): Transaction | null;

declare function autocompletion(config?: CompletionConfig): Extension;
declare const completionKeymap: readonly KeyBinding[];
declare function completionStatus(state: EditorState): null | "active" | "pending";
declare function currentCompletions(state: EditorState): readonly Completion[];
declare function selectedCompletion(state: EditorState): Completion | null;
declare function selectedCompletionIndex(state: EditorState): number | null;
declare function setSelectedCompletion(index: number): StateEffect<unknown>;

export { CloseBracketConfig, Completion, CompletionContext, CompletionResult, CompletionSection, CompletionSource, acceptCompletion, autocompletion, clearSnippet, closeBrackets, closeBracketsKeymap, closeCompletion, completeAnyWord, completeFromList, completionKeymap, completionStatus, currentCompletions, deleteBracketPair, ifIn, ifNotIn, insertBracket, insertCompletionText, moveCompletionSelection, nextSnippetField, pickedCompletion, prevSnippetField, selectedCompletion, selectedCompletionIndex, setSelectedCompletion, snippet, snippetCompletion, snippetKeymap, startCompletion };
